@@ -1,65 +1,27 @@
 # -*- test-case-name: openid.test.test_xri -*-
 """Utility functions for handling XRIs.
 
-@see: XRI Syntax v2.0 at the U{OASIS XRI Technical Committee<http://www.oasis-open.org/committees/tc_home.php?wg_abbrev=xri>}
+@see: XRI Syntax v2.0 at the
+      U{OASIS XRI Technical Committee<http://www.oasis-open.org/committees/tc_home.php?wg_abbrev=xri>}
 """
+from __future__ import unicode_literals
 
 import re
 
+from six.moves.urllib.parse import quote
+
+from openid.oidutil import string_to_text
+from openid.urinorm import GEN_DELIMS, PERCENT_ENCODING_CHARACTER, SUB_DELIMS
+
 XRI_AUTHORITIES = ['!', '=', '@', '+', '$', '(']
 
-try:
-    unichr(0x10000)
-except ValueError:
-    # narrow python build
-    UCSCHAR = [
-        (0xA0, 0xD7FF),
-        (0xF900, 0xFDCF),
-        (0xFDF0, 0xFFEF),
-        ]
-
-    IPRIVATE = [
-        (0xE000, 0xF8FF),
-        ]
-else:
-    UCSCHAR = [
-        (0xA0, 0xD7FF),
-        (0xF900, 0xFDCF),
-        (0xFDF0, 0xFFEF),
-        (0x10000, 0x1FFFD),
-        (0x20000, 0x2FFFD),
-        (0x30000, 0x3FFFD),
-        (0x40000, 0x4FFFD),
-        (0x50000, 0x5FFFD),
-        (0x60000, 0x6FFFD),
-        (0x70000, 0x7FFFD),
-        (0x80000, 0x8FFFD),
-        (0x90000, 0x9FFFD),
-        (0xA0000, 0xAFFFD),
-        (0xB0000, 0xBFFFD),
-        (0xC0000, 0xCFFFD),
-        (0xD0000, 0xDFFFD),
-        (0xE1000, 0xEFFFD),
-        ]
-
-    IPRIVATE = [
-        (0xE000, 0xF8FF),
-        (0xF0000, 0xFFFFD),
-        (0x100000, 0x10FFFD),
-        ]
-
-
-_escapeme_re = re.compile('[%s]' % (''.join(
-    map(lambda (m, n): u'%s-%s' % (unichr(m), unichr(n)),
-        UCSCHAR + IPRIVATE)),))
-
 
 def identifierScheme(identifier):
     """Determine if this identifier is an XRI or URI.
 
     @returns: C{"XRI"} or C{"URI"}
     """
-    if identifier.startswith('xri://') or identifier[0] in XRI_AUTHORITIES:
+    if identifier.startswith('xri://') or (identifier and identifier[0] in XRI_AUTHORITIES):
         return "XRI"
     else:
         return "URI"
@@ -97,15 +59,24 @@ def toURINormal(xri):
     return iriToURI(toIRINormal(xri))
 
 
-def _percentEscapeUnicode(char_match):
-    c = char_match.group()
-    return ''.join(['%%%X' % (ord(octet),) for octet in c.encode('utf-8')])
-
-
 def iriToURI(iri):
-    """Transform an IRI to a URI by escaping unicode."""
-    # According to RFC 3987, section 3.1, "Mapping of IRIs to URIs"
-    return _escapeme_re.sub(_percentEscapeUnicode, iri)
+    """Transform an IRI to a URI by escaping unicode.
+
+    According to RFC 3987, section 3.1, "Mapping of IRIs to URIs"
+
+    @type iri: six.text_type, six.binary_type deprecated.
+    @rtype: six.text_type
+    """
+    iri = string_to_text(iri, "Binary input for iriToURI is deprecated. Use text input instead.")
+
+    # This is hackish. `quote` requires `str` in both py27 and py3+.
+    if isinstance(iri, str):
+        # Python 3 branch
+        return quote(iri, GEN_DELIMS + SUB_DELIMS + PERCENT_ENCODING_CHARACTER)
+    else:
+        # Python 2 branch
+        return quote(iri.encode('utf-8'),
+                     (GEN_DELIMS + SUB_DELIMS + PERCENT_ENCODING_CHARACTER).encode('utf-8')).decode('utf-8')
 
 
 def providerIsAuthoritative(providerID, canonicalID):
@@ -126,8 +97,8 @@ def rootAuthority(xri):
 
         rootAuthority("xri://@example") == "xri://@"
 
-    @type xri: unicode
-    @returntype: unicode
+    @type xri: six.text_type
+    @returntype: six.text_type
     """
     if xri.startswith('xri://'):
         xri = xri[6:]
@@ -145,8 +116,7 @@ def rootAuthority(xri):
     else:
         # IRI reference.  XXX: Can IRI authorities have segments?
         segments = authority.split('!')
-        segments = reduce(list.__add__,
-            map(lambda s: s.split('*'), segments))
+        segments = [c for s in segments for c in s.split('*')]
         root = segments[0]
 
     return XRI(root)
@@ -160,7 +130,7 @@ def XRI(xri):
     canonicalization by ensuring the xri scheme is present.
 
     @param xri: an xri string
-    @type xri: unicode
+    @type xri: six.text_type
     """
     if not xri.startswith('xri://'):
         xri = 'xri://' + xri
